/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SORT_HPP
#define SORT_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/

struct SortNameAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getName() < second.getFile().getName();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};

struct SortNameDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getName() > second.getFile().getName();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};


struct SortSizeAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getSize() < second.getFile().getSize();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};


struct SortSizeDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getSize() > second.getFile().getSize();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};


struct SortStringFieldAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getFields().at(sortField) < second.getFile().getFields().at(sortField);
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
     
     SortStringFieldAscending(int _sortField)
     : sortField(_sortField){}
private:
	int sortField;     
     
};


struct SortStringFieldDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getFields().at(sortField) > second.getFile().getFields().at(sortField);
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
     
     SortStringFieldDescending(int _sortField)
     : sortField(_sortField){}
private:
	int sortField;     
     
};


struct SortMTimeAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getModificationTime() < second.getFile().getModificationTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};

struct SortMTimeDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getModificationTime() > second.getFile().getModificationTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};


struct SortATimeAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getAccessTime() < second.getFile().getAccessTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};

struct SortATimeDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getAccessTime() > second.getFile().getAccessTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};

struct SortCTimeAscending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getCreationTime() < second.getFile().getCreationTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};

struct SortCTimeDescending
{
     bool operator()(const FileIcon& first, const FileIcon& second)
     {
     	  if(!(first.getFile().isDirectory() - second.getFile().isDirectory()))
          return first.getFile().getCreationTime() > second.getFile().getCreationTime();
          else if(first.getFile().isDirectory())
          return true;
          else
          return false;
          
     }
};


#endif
