/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef VFS_H
#define VFS_H

#include "File.hpp"
#include "Status.hpp"
#include "Question.hpp"
#include "QuestionManager.hpp"
#include "Directory.hpp"
#include "Panel.hpp"
#include "Cancelable.hpp"

#include <boost/filesystem/path.hpp>

#include <iostream>
#include <vector>
using namespace std;
using namespace boost::filesystem;

class Directory;

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class Vfs
{
public:
	virtual Directory openDir( path &p, Cancelable *cancelable=NULL ) = 0;    
	virtual File readFile(path file)=0;
	virtual void createDir(path p) = 0;
	
	virtual void copyFromFile(File &source, path &destination, Status * status, Cancelable *cancelable)=0;
	virtual void copyToFile(File &source, path &destination, Status * status, Cancelable *cancelable)=0;
	virtual void copy(Vfs * vfs2,path source, path destination,Status * status, Cancelable *cancelable)=0;
	
	virtual path symlink(const path& file)=0;    
	virtual double getTotalSize(path &p, Cancelable *cancelable)=0;
	virtual string getType()=0;
	virtual void init(Panel *panel,QuestionManager *questionManager)=0;
	virtual ~Vfs(){}
	virtual void chmod(path &source,unsigned int mode)=0;
	virtual void chmodRecursive(path &source,unsigned int mode)=0;
	virtual void removeFile(path &source)=0;
    	virtual void removeDirectory(path &source)=0;
    	virtual void remove(path &source)=0;
    	virtual void rename(path &source, path &destination)=0;
    	virtual File readFullFile(File file)=0;
    	virtual void changeOwnerRecursive(path &source,string user)=0;
    	virtual void changeOwner(path &source,string user)=0;
    	virtual void changeGroupRecursive(path &source,string group)=0;
    	virtual void changeGroup(path &source,string group)=0;
    	virtual vector<string> getUsers()=0;
    	virtual vector<string> getGroups()=0;
    	virtual void createSymlink(path &source,path &destination)=0;
    
private:
	friend class Directory;
	friend class VfsSync;
	virtual File readDir( Directory *directory) = 0;  
	virtual void closeDir( Directory *directory) = 0;  

};

#endif
