/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef WINDOWCONTENT_HPP
#define WINDOWCONTENT_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/

#include <string>
#include <vector>
#include <iostream>
using namespace std;

class Widget
{
public:

	void setParentName ( const string& theValue )
	{
		parentName = theValue;
	}
	

	string getParentName() const
	{
		return parentName;
	}

	void setName ( const string& theValue )
	{
		name = theValue;
	}
	

	string getName() const
	{
		return name;
	}

	void setType ( const string& theValue )
	{
		type = theValue;
	}
	

	string getType() const
	{
		return type;
	}
	
	virtual ~Widget(){}
	
protected:	
	
	string parentName;
	string name;
	string type;

};

class WidgetVerticalFrame : public Widget
{
public:

	WidgetVerticalFrame(string _parentName, string _name) {setParentName(_parentName);setName(_name);setType("VerticalFrame");}	
	WidgetVerticalFrame(){}
};


class WidgetHorizontalFrame : public Widget
{
public:

	WidgetHorizontalFrame(string _parentName, string _name){setParentName(_parentName);setName(_name);setType("HorizontalFrame");}	
	WidgetHorizontalFrame(){}
};

class WidgetGroupBox : public Widget
{
public:

	WidgetGroupBox(string _parentName, string _name, string _value) {setParentName(_parentName);setName(_name);setType("GroupBox");setValue(_value);}	
	WidgetGroupBox(){}

	void setValue ( const string& theValue )
	{
		value = theValue;
	}
	

	string getValue() const
	{
		return value;
	}
	
private:
	string value;	
};


class WidgetCheckButton : public Widget
{
public:

	WidgetCheckButton(string _parentName, string _name, string _label, bool _value) {setParentName(_parentName);setName(_name);setType("CheckButton");setLabel(_label);setValue(_value);}	
	WidgetCheckButton(){}

	void setValue ( bool theValue )
	{
		value = theValue;
	}
	

	bool getValue() const
	{
		return value;
	}

	void setLabel ( const string& theValue )
	{
		label = theValue;
	}
	

	string getLabel() const
	{
		return label;
	}
	
	


	
private:
	string label;
	bool value;	
};


class WidgetLabel : public Widget
{
public:

	WidgetLabel(string _parentName, string _name,string _value, string _iconName)
	{setParentName(_parentName);setName(_name);setValue(_value);setIconName(_iconName);setType("Label");}	
	WidgetLabel(){}
	
	void setValue ( const string& theValue )
	{
		value = theValue;
	}
	

	string getValue() const
	{
		return value;
	}
	
	

	void setIconName ( const string& theValue )
	{
		iconName = theValue;
	}
	

	string getIconName() const
	{
		return iconName;
	}
	
private:
	string value;
	string iconName;
	
};

class WidgetTextField : public Widget
{
public:

	WidgetTextField(string _parentName, string _name,string _value,int fieldsNum, string style)
	{setParentName(_parentName);setName(_name);setValue(_value);setType("TextField");setFieldsNum(fieldsNum);setStyle(style);}	
	WidgetTextField(){}
	
	void setValue ( const string& theValue )
	{
		value = theValue;
	}
	

	string getValue() const
	{
		return value;
	}

	void setFieldsNum ( int theValue )
	{
		fieldsNum = theValue;
	}
	

	int getFieldsNum() const
	{
		return fieldsNum;
	}

	void setStyle ( const string& theValue )
	{
		style = theValue;
	}
	

	string getStyle() const
	{
		return style;
	}
	


	
	
	
private:
	string value;
	int fieldsNum;
	string style;
};

class WidgetButton : public Widget
{
public:

	WidgetButton(string _parentName, string _name,string _value, string _iconName)
	{setParentName(_parentName);setName(_name);setValue(_value);setIconName(_iconName);setType("Button");}	
	WidgetButton(){}
	
	void setValue ( const string& theValue )
	{
		value = theValue;
	}
	

	string getValue() const
	{
		return value;
	}
	
	

	void setIconName ( const string& theValue )
	{
		iconName = theValue;
	}
	

	string getIconName() const
	{
		return iconName;
	}
	
private:
	string value;
	string iconName;
};

class WidgetComboBox : public Widget
{
public:

	WidgetComboBox(string _parentName, string _name,vector<string> _values,int _fieldsNum)
	{setParentName(_parentName);setName(_name);setValues(_values);setFieldsNum(_fieldsNum);setType("ComboBox");}	
	WidgetComboBox(){}

	void setValues ( const vector< string >& theValue )
	{
		values = theValue;
	}
	

	vector< string > getValues() const
	{
		return values;
	}

	void setFieldsNum ( int theValue )
	{
		fieldsNum = theValue;
	}
	

	int getFieldsNum() const
	{
		return fieldsNum;
	}
		

	
private:
	vector<string> values;
	int fieldsNum;

};




class WindowContent
{
public:
	WindowContent();
	~WindowContent();
	
	void setSize(int width, int height);
	void addWidgetVerticalFrame(string parentName,string name);
	void addWidgetHorizontalFrame(string parentName,string name);
	void addWidgetGroupBox(string parentName, string name, string value="");
	void addWidgetCheckButton(string parentName, string name, string label="",bool value=false);
	void addWidgetLabel(string parentName,string name, string value="", string iconName="");
	void addWidgetTextField(string parentName,string name, string value="",int fieldsNum=0,string style="");
	void addWidgetButton(string parentName,string name, string value="", string iconName="");
	void addWidgetComboBox(string parentName,string name, vector<string> values,int fieldsNum=0);
	
	Widget *getNext();

	int getWidth() const
	{
		return width;
	}

	int getHeight() const
	{
		return height;
	}

	void setWidgetFocus ( const string& theValue )
	{
		widgetFocus = theValue;
	}
	

	string getWidgetFocus() const
	{
		return widgetFocus;
	}

	void setTitle ( const string& theValue )
	{
		title = theValue;
	}
	

	string getTitle() const
	{
		return title;
	}

	void setDefaultButton ( const string& theValue )
	{
		defaultButton = theValue;
	}
	

	string getDefaultButton() const
	{
		return defaultButton;
	}
	
	
	
	
	
    
    
private:
	vector<Widget*>widgets; 
	int count;
	int width;
	int height;
	string widgetFocus;
	string title;
	string defaultButton;

};

#endif
