/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "StandardCommands.hpp"
#include "Settings.hpp"
#include "FilesEngine.hpp"
#include "Exceptions.hpp"
#include "Iconv.hpp"

#include <boost/regex.hpp>
#include <sys/mount.h>

#include "config.h"
#include "gettext.h"
#define _(STRING)    gettext(STRING)



REGISTER_FACTORABLE(CmdSmallIconsView);
REGISTER_FACTORABLE(CmdBigIconsView);
REGISTER_FACTORABLE(CmdDetailsIconsView);
REGISTER_FACTORABLE(CmdSwitchPanelsView);
REGISTER_FACTORABLE(CmdGotoOppositePanel);
REGISTER_FACTORABLE(CmdOpenOpposite);
REGISTER_FACTORABLE(CmdSelectAll);
REGISTER_FACTORABLE(CmdTotalSize);
REGISTER_FACTORABLE(CmdShowPreview);
REGISTER_FACTORABLE(CmdOpenDirectory);
REGISTER_FACTORABLE(CmdOpenPreviousInHistory);
REGISTER_FACTORABLE(CmdEnlargePreview);
REGISTER_FACTORABLE(CmdReducePreview);
REGISTER_FACTORABLE(CmdStopThreads);
REGISTER_FACTORABLE(CmdOpenLocalVfs);
REGISTER_FACTORABLE(CmdOpenRemoteVfs);
REGISTER_FACTORABLE(CmdChmod);
REGISTER_FACTORABLE(CmdRemove);
REGISTER_FACTORABLE(CmdRename);
REGISTER_FACTORABLE(CmdOpenParentDirectory);
REGISTER_FACTORABLE(CmdOpenHomeDirectory);
REGISTER_FACTORABLE(CmdRefresh);
REGISTER_FACTORABLE(CmdShowHideHiddenFiles);
REGISTER_FACTORABLE(CmdChangeOwner);
REGISTER_FACTORABLE(CmdChangeGroup);
REGISTER_FACTORABLE(CmdCreateSymlink);
REGISTER_FACTORABLE(CmdClone);
REGISTER_FACTORABLE(CmdGetFileInformation);
REGISTER_FACTORABLE(CmdClipboardCopy);
REGISTER_FACTORABLE(CmdClipboardCut);
REGISTER_FACTORABLE(CmdClipboardPaste);
REGISTER_FACTORABLE(CmdOpenPath);
REGISTER_FACTORABLE(CmdRunCommand);
REGISTER_FACTORABLE(CmdOpenFlatDirectory);
REGISTER_FACTORABLE(CmdOpenHistoryVfs);
REGISTER_FACTORABLE(CmdOpenSearchVfs);
REGISTER_FACTORABLE(CmdOpenArchiveVfs);
REGISTER_FACTORABLE(CmdRegexSelection);
REGISTER_FACTORABLE(CmdRegexFilter);
REGISTER_FACTORABLE(CmdAddDirToShortcuts);
REGISTER_FACTORABLE(CmdUmount);
REGISTER_FACTORABLE(CmdCreateEmptyFile);


void CmdSmallIconsView::execute( )
{
	fileList->getFileListIcons()->setView( "small icons" );
	singleton<Settings>::instance().setFileListStyle(fileList->getVfsName(),"small icons");
}

void CmdBigIconsView::execute( )
{
	fileList->getFileListIcons()->setView( "big icons" );
	singleton<Settings>::instance().setFileListStyle(fileList->getVfsName(),"big icons");
}

void CmdDetailsIconsView::execute( )
{
	fileList->getFileListIcons()->setView( "details" );
	singleton<Settings>::instance().setFileListStyle(fileList->getVfsName(),"details");
}



void CmdSwitchPanelsView::execute( )
{
	Settings &settings=singleton<Settings>::instance();
	settings.setSinglePanel(!settings.isSinglePanel());	
	fileList->getFileListIcons()->switchPanelsView();

}

void CmdGotoOppositePanel::execute( )
{
	fileListOpposite->getFileListIcons()->setFocus();
}

void CmdOpenOpposite::execute()
{
	fileList->openDirectory(fileListOpposite->getCurrentDirectory());	
}

void CmdSelectAll::execute()
{
	fileList->getFileListIcons()->selectAll();
}

void CmdTotalSize::execute()
{
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;
	double size=0;
	double currentSize=0;
	Settings &settings=singleton<Settings>::instance();
	vector<string> headers;
	vector<string>::iterator iter2;	
	headers=settings.getHeaders("Local");
	int headerSizePosition=0;
	for(iter2=headers.begin();iter2!=headers.end();++iter2)
	{
		if(*iter2=="size")
		break;
		
		++headerSizePosition;
	}
	
	
	
	for(iter=files.begin();iter!=files.end();++iter)
	{
		currentSize=fileList->getVfs()->getTotalSize(*iter,this);
		size+=currentSize;
		/*TODO*/

		FileIcon fileIcon;
		
		try
		{
		fileIcon=fileList->getFileIcon(iter->string());
		}
		catch(ExceptionFileIconNotFound &e)
		{
		return;
		}
		
		File file=fileIcon.getFile();
		vector<string>fields=file.getFields();
		/*first position is name */
		fields[headerSizePosition-1]=FilesEngine::printSize(currentSize);
		file.setFields(fields);
		fileIcon.setFile(file);
		
			try
			{
			fileList->replaceFileIcon(fileIcon);
			}
			catch(ExceptionFileIconNotFound &e)
			{
			return;
			}
			
		if(isCanceledExecution())		
		return;
		
	}
	fileList->getFileListIcons()->setStatusInfo(FilesEngine::printSize(size));
	
}




void CmdShowPreview::execute()
{
	/*TODO*/
	
	
	if(fileList->getVfsType()!="Local")
	return;
	
	Settings &settings=singleton<Settings>::instance();
	
	//if(settings.getFileListStyle(fileList->getVfsName())!="big icons")
	//return;
	
	
	int previewSize=settings.getPreviewSize();
	
	FileListIcons *fileListIcons=fileList->getFileListIcons();
	
	if(getParam()!="resize")
	{	
		for(int i=0;i<10;i++)
		{
			if(isCanceledExecution())
			return;
		usleep(100000);
		}
	fileListIcons->unloadPreviewIcons();
	}
	
	vector<string> allFiles=fileList->getAllFilesPaths();
	vector<string>::iterator iter;
	
	for(iter=allFiles.begin();iter!=allFiles.end();++iter)
	{
		if(isCanceledExecution())		
		{
		break;
		}

		
		FileIcon fileIcon=fileList->getFileIcon(iter->c_str());
		File file=fileIcon.getFile();
		
		string mime=file.getMime();
		
		double maxPrieviewFileSize=settings.getMaxThumbSize();
		if(file.getSize()<maxPrieviewFileSize && mime.size()>5 && mime.substr(0,5)=="image")
		{
			
			int position=fileListIcons->loadPreviewIcon(file.getPath().string(),previewSize);
			if(position!=-1)
			{
			fileIcon.setPreviewIcon(position);
				
				try
				{
				fileList->replaceFileIcon(fileIcon);
				}
				catch(ExceptionFileIconNotFound &e)
				{
				break;
				}
			}
		}
		
		
	}
}

void CmdOpenDirectory::execute()
{
	Settings &settings=singleton<Settings>::instance();
	bool showHidden=settings.getShowHiddenFiles();

	path p=fileList->getNewDirectory();
	vector<File>files;
		try
		{
			
			Vfs*vfs=fileList->getVfs();
			Directory directory=vfs->openDir( p, this );	
								
			fileList->removeFileIcons();
			
			while(1)
			{
			
			File file=directory.getNextFile();
			
			
				if(file.getName()==".")
				continue;
			
			
				if(!showHidden && file.hidden())
				continue;
			
			
			file=vfs->readFullFile(file);
			files.push_back(file);
			
					
			}		
		
		}
		catch(ExceptionReadDirEnd &e)
		{
			fileList->setFiles(files);
			fileList->sort();
			fileList->setCurrentDirectory(p);
			
			
			if(settings.getMaxThumbSize()>0)
			fileList->runCommand("ShowPreview");
		}
		catch(ExceptionDirOpen &e)
		{
			Question::displayError(fileList->getFileListIcons(),_("can't open"));
		}
	
}

void CmdOpenPreviousInHistory::execute()
{
//using list...because of boost python which can't handle vector :/
	vector<string>& history=fileList->getPathHistory();
	if(history.size()>1)
	{		
		history.pop_back();
		string dir=history.back();
		history.pop_back();
		fileList->openDirectory(path(dir));
		
		/*TODO add setting history size*/
		if(history.size()>50)
		{
//			history.pop_front();
			history.erase(history.begin());
		}
		
	}
}

void CmdEnlargePreview::execute()
{
	singleton<Settings>::instance().setPreviewSize(singleton<Settings>::instance().getPreviewSize()+16);
	fileList->runCommand("ShowPreview","resize");
}

void CmdReducePreview::execute()
{
	singleton<Settings>::instance().setPreviewSize(singleton<Settings>::instance().getPreviewSize()-16);
	fileList->runCommand("ShowPreview","resize");
}

void CmdStopThreads::execute()
{
}	

/****************************************************************************/

void CmdOpenLocalVfs::execute()
{		
	fileList->getFileListIcons()->createPanel("FileList",getParam());
}

/****************************************************************************/


void CmdOpenRemoteVfs::execute()
{
		map<string,string>params=FilesEngine::parseParam(getParam());
		
		if(params["hostname"]!="" && params["user"]!="")
		{
			fileList->getFileListIcons()->createPanel("FileList",getParam());
		}
		else
		{
		
		Settings &settings=singleton<Settings>::instance();
				
		
		WindowContent windowContent;
		windowContent.setSize(320,190);
		windowContent.addWidgetVerticalFrame("","vframe1");
		
		vector<string>hostnames=settings.getHistoryRemoteVfsHostnames();
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		windowContent.addWidgetComboBox("hframe1","hostname",hostnames,25);
		windowContent.addWidgetLabel("hframe1","hostnameLabel",_("<= hostname"));
		
		vector<string>users=settings.getHistoryRemoteVfsUsers();
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe2");
		windowContent.addWidgetComboBox("hframe2","user",users,25);
		windowContent.addWidgetLabel("hframe2","userLabel",_("<=  user"));
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe3");
		windowContent.addWidgetTextField("hframe3","password","",25,"encrypt");	
		windowContent.addWidgetLabel("hframe3","passwordLabel",_("<= password"));
			
		windowContent.addWidgetHorizontalFrame("vframe1","hframe4");
		windowContent.addWidgetTextField("hframe4","port","",6,"integer");	
		windowContent.addWidgetLabel("hframe4","portLabel",_("<= port"));
		
		vector<string>vfsList;
		vfsList.push_back("Ftp");
		vfsList.push_back("Sftp");
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe8");
		windowContent.addWidgetComboBox("hframe8","vfs",vfsList,25);
		windowContent.addWidgetLabel("hframe8","vfsLabel",_("<= vfs"));
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetButton("hframe10","ok",_("ok"),"ok");
		windowContent.addWidgetButton("hframe10","cancel",_("cancel"),"cancel");	
		
		windowContent.setDefaultButton("ok");
		
		map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);
	
		string newParams="vfs="+answer["vfs"]+":hostname="+answer["hostname"]+":user="+answer["user"]+":port="+answer["port"]+":password="+answer["password"];
		

			if(answer["pressedButton"]=="ok" && answer["hostname"]!="")
			{
				if(hostnames.end()==find(hostnames.begin(),hostnames.end(),answer["hostname"]))
				{
				hostnames.push_back(answer["hostname"]);
				settings.setHistoryRemoteVfsHostnames(hostnames);					
				}
				
				if(answer["user"]!="")
				{
					if(users.end()==find(users.begin(),users.end(),answer["user"]))
					{	
					users.push_back(answer["user"]);		
					settings.setHistoryRemoteVfsUsers(users);
					}
				}
				
				fileList->getFileListIcons()->createPanel("FileList",newParams);
			}
		
		
		}

}

void CmdChmod::execute()
{


	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;
	double size=0;
	for(iter=files.begin();iter!=files.end();++iter)
	{	
	
		File file=fileList->getFileIcon(iter->string()).getFile();
	
		if(isCanceledExecution())		
		return;

		WindowContent windowContent;
		windowContent.setSize(360,190);
		windowContent.addWidgetVerticalFrame("","vframe1");
		
		windowContent.addWidgetLabel("vframe1","fileName",_("File: ")+Iconv::convertToUTF8IfNeeded(file.getName()),fileList->getFileIcon(iter->string()).getIconName());
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		windowContent.addWidgetGroupBox("hframe1","group1",_("user"));
		windowContent.addWidgetCheckButton("group1","user_r",_("read"),file.isOwnerReadable());
		windowContent.addWidgetCheckButton("group1","user_w",_("write"),file.isOwnerWritable());
		windowContent.addWidgetCheckButton("group1","user_x",_("execute"),file.isOwnerExecutable());
		
		windowContent.addWidgetGroupBox("hframe1","group2",_("group"));
		windowContent.addWidgetCheckButton("group2","group_r",_("read"),file.isGroupReadable());
		windowContent.addWidgetCheckButton("group2","group_w",_("write"),file.isGroupWritable());
		windowContent.addWidgetCheckButton("group2","group_x",_("execute"),file.isGroupExecutable());
		
		windowContent.addWidgetGroupBox("hframe1","group3",_("other"));
		windowContent.addWidgetCheckButton("group3","other_r",_("read"),file.isOtherReadable());
		windowContent.addWidgetCheckButton("group3","other_w",_("write"),file.isOtherWritable());
		windowContent.addWidgetCheckButton("group3","other_x",_("execute"),file.isOtherExecutable());
		
		
		windowContent.addWidgetGroupBox("hframe1","group4",_("special"));
		windowContent.addWidgetCheckButton("group4","setuid",_("set UID"),file.isUID());
		windowContent.addWidgetCheckButton("group4","setgid",_("set GID"),file.isGID());
		windowContent.addWidgetCheckButton("group4","sticky",_("sticky"),file.isSticky());
		
		if(file.isDirectory())
		windowContent.addWidgetCheckButton("vframe1","recursive",_("recursive"),false);
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetButton("hframe10","buttonOk",_("ok"),"ok");
		windowContent.addWidgetButton("hframe10","buttonCancel",_("cancel"),"cancel");
		
	
		map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);
		
		if(answer["pressedButton"]=="cancel")
		return;
		
		file.setMode(0);
		
		if(answer["user_r"]=="true")
			file.setOwnerReadable();
		if(answer["user_w"]=="true")
			file.setOwnerWritable();	
		if(answer["user_x"]=="true")
			file.setOwnerExecutable();	
		if(answer["group_r"]=="true")
			file.setGroupReadable();
		if(answer["group_w"]=="true")
			file.setGroupWritable();	
		if(answer["group_x"]=="true")
			file.setGroupExecutable();
		if(answer["other_r"]=="true")
			file.setOtherReadable();
		if(answer["other_w"]=="true")
			file.setOtherWritable();	
		if(answer["other_x"]=="true")
			file.setOtherExecutable();	
		if(answer["setuid"]=="true")
			file.setUID();
		if(answer["setgid"]=="true")
			file.setGID();		
		if(answer["sticky"]=="true")
			file.setSticky();
					
		path p=file.getPath();		
				
		if(answer["recursive"]=="true")
			fileList->getVfs()->chmodRecursive(p,file.getMode());
		else		
			fileList->getVfs()->chmod(p,file.getMode());
		
		
		fileList->refreshFileIcon(file.getPath().string());
		
		
	}	
		
}

void CmdRemove::execute()
{
	
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;

	bool all=false;

	for(iter=files.begin();iter!=files.end();++iter)
	{

		
		if(!all)
		{
				
		bool many=false;						
		if(files.size()>1)
		many=true;		
				
		string answer=Question::askOverwrite(fileList->getFileListIcons(),fileList->getFileIcon(iter->string()).getFile(),_("remove:"),many);
					
					if(answer=="cancel")
					{
						break;
					}
					else if(answer=="skip")
					{
						continue;
					}
					else if(answer=="skip_all")
					{
						break;
					}
					else if(answer=="all")
					{
						all=true;					
					}		
		}
		

		fileList->getVfs()->remove(*iter);
		
				try
				{
				fileList->removeFileIcon(iter->string());
				}
				catch(ExceptionFileIconNotFound &e)
				{
				break;
				}
			
		if(isCanceledExecution())		
		return;
		
	}
}

void CmdRename::execute()
{


	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;

	for(iter=files.begin();iter!=files.end();++iter)
	{
		try
		{
		string name=iter->leaf();
		
		bool converted=false;
		if(!Iconv::isUTF8(name))
		{		
			name=Iconv::toUTF8(iter->leaf());
			converted=true;
		}
		string askString;
		
		#ifdef  HAVE_LIBFOX_1_6
			if(converted)
			askString=_("new name (this file doesn't have UTF8 filename):");
			else
			askString=_("new name:");
		#else
		askString=_("new name:");
		#endif
		
		string newName=Question::askString(fileList->getFileListIcons(),askString,name);
		
			if(newName!="")
			{		
		
			
			path newPath;
			
			if(!converted || !singleton<Settings>::instance().getSaveOriginalCharsetOnRename())
			newPath=iter->branch_path()/path(newName);
			else
			{
			newPath=iter->branch_path()/path(Iconv::fromUTF8(newName));
			//cout<<Iconv::fromUTF8(newName)<<endl;
			//cout<<newName<<endl;
			}
			
			fileList->getVfs()->rename(*iter,newPath);
			
				try
				{
				fileList->removeFileIcon(iter->string());
				fileList->addFile(newPath.string());
				}
				catch(ExceptionFileIconNotFound &e)
				{
				break;
				}
			
			}
		}
		catch(ExceptionCancel &e){}
			
			
		if(isCanceledExecution())		
		return;
		
	}
}

void CmdOpenParentDirectory::execute()
{
	fileList->openParentDir();
}

void CmdRefresh::execute()
{
	fileList->reloadDirectory();
}

void CmdOpenHomeDirectory::execute()
{
	fileList->openDirectory(FilesEngine::getHomeDir());
}

void CmdShowHideHiddenFiles::execute()
{
	Settings &settings=singleton<Settings>::instance();
	settings.setShowHiddenFiles(!settings.getShowHiddenFiles());
	fileList->reloadDirectory();
}


void CmdChangeOwner::execute()
{
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;

	
	if(FilesEngine::getCurrentUser()!="root")
	{
	Question::displayError(fileList->getFileListIcons(),_("You don't have enough privileges"));
	return;
	}

	for(iter=files.begin();iter!=files.end();++iter)
	{	
	
		File file=fileList->getFileIcon(iter->string()).getFile();
	
		if(isCanceledExecution())		
		return;

		WindowContent windowContent;
		windowContent.addWidgetVerticalFrame("","vframe1");
		
		windowContent.addWidgetLabel("vframe1","fileName","File: "+Iconv::convertToUTF8IfNeeded(file.getName()),fileList->getFileIcon(iter->string()).getIconName());
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		vector<string>users=fileList->getVfs()->getUsers();
		
		sort(users.begin(),users.end());		
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		windowContent.addWidgetComboBox("hframe1","users",users,25);
		
		if(file.isDirectory())
		windowContent.addWidgetCheckButton("vframe1","recursive",_("recursive"),false);
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetButton("hframe10","ok",_("ok"),"ok");
		windowContent.addWidgetButton("hframe10","cancel",_("cancel"),"cancel");
		
	
		map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);
		
		if(answer["pressedButton"]=="cancel")
		return;
		
		string newuser=answer["users"];
		
					
		path p=file.getPath();		
				
		try
		{		
		if(answer["recursive"]=="true")
			fileList->getVfs()->changeOwnerRecursive(p,newuser);
		else		
			fileList->getVfs()->changeOwner(p,newuser);
		}
		catch(ExceptionFileSystem &e)
		{
			Question::displayException(fileList->getFileListIcons(),e);
		}	
		
		try
		{
		fileList->refreshFileIcon(file.getPath().string());
		}
		catch(ExceptionFileIconNotFound &e)
		{
		break;
		}
		
		
	}	
		
}

void CmdChangeGroup::execute()
{
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;

	for(iter=files.begin();iter!=files.end();++iter)
	{
		
		File file=fileList->getFileIcon(iter->string()).getFile();
	
		if(isCanceledExecution())		
		return;
	
		WindowContent windowContent;
		windowContent.addWidgetVerticalFrame("","vframe1");
		
		windowContent.addWidgetLabel("vframe1","fileName",_("File: ")+Iconv::convertToUTF8IfNeeded(file.getName()),fileList->getFileIcon(iter->string()).getIconName());
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		
			
		vector<string>groups=fileList->getVfs()->getGroups();
		
		//vector<string>::iterator stringIter;
		
		if(fileList->getVfsType()=="Local")
		{
			if(FilesEngine::getCurrentUser()!="root")
			groups=FilesEngine::getCurrentGroups();
			/*
			vector<string>currentGroups=FilesEngine::getCurrentGroups();
			for(stringIter=allGroups.begin();stringIter!=allGroups.end();++stringIter)
			{
				if(find(currentGroups.begin(),currentGroups.end(),*stringIter)!=currentGroups.end())
				{
					groups.push_back(*stringIter);
				}
			}
			*/
			
			if(FilesEngine::getCurrentUser()!="root")
			{
				if(find(groups.begin(),groups.end(),file.getGroup())==groups.end())
				{
				Question::displayError(fileList->getFileListIcons(),_("You don't have enough privileges"));
				return;
				}
			}
		}
		
		sort(groups.begin(),groups.end());
		
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		windowContent.addWidgetComboBox("hframe1","groups",groups,25);
		
		if(file.isDirectory())
		windowContent.addWidgetCheckButton("vframe1","recursive",_("recursive"),false);
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetButton("hframe10","ok",_("ok"),"ok");
		windowContent.addWidgetButton("hframe10","cancel",_("cancel"),"cancel");
		
	
		map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);
		
		if(answer["pressedButton"]=="cancel")
		return;
		
		string newgroup=answer["groups"];
		
					
		path p=file.getPath();		
				
		if(answer["recursive"]=="true")
			fileList->getVfs()->changeGroupRecursive(p,newgroup);
		else		
			fileList->getVfs()->changeGroup(p,newgroup);
		
		try
		{
		fileList->refreshFileIcon(file.getPath().string());
		}
		catch(ExceptionFileIconNotFound &e)
		{
		break;
		}
		
		
	}
}




void CmdCreateSymlink::execute()
{
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;

	
	

	if(fileListOpposite->getVfsType()!=fileList->getVfsType())
	return;

	for(iter=files.begin();iter!=files.end();++iter)
	{
		

		try
		{
		string newName=Question::askString(fileList->getFileListIcons(),string(iter->leaf()));
			if(newName!="")
			{
			path newPath=fileListOpposite->getCurrentDirectory()/path(newName);
			path oldpath=*iter;
			path newpath=newPath;
			
			fileList->getVfs()->createSymlink(oldpath,newpath);
			
			try
			{
			fileList->addFile(newPath.string());
			}
			catch(ExceptionFileIconNotFound &e)
			{
			break;
			}
		
			
			}
		}
		catch(ExceptionCancel &e){}
	
		if(isCanceledExecution())		
		return;
		
	}	
	
}




void CmdClone::execute()
{
	
}

void CmdGetFileInformation::execute()
{
	vector<path>files=fileList->getSelectedFileIconsPaths();
	vector<path>::iterator iter;


	for(iter=files.begin();iter!=files.end();++iter)
	{
	
		File file=fileList->getFileIcon(iter->string()).getFile();
		
		WindowContent windowContent;
		windowContent.addWidgetVerticalFrame("","vframe1");
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
		windowContent.addWidgetLabel("hframe1","name",_("name: ")+Iconv::convertToUTF8IfNeeded(file.getName()));	

		windowContent.addWidgetHorizontalFrame("vframe1","hframe2");
		windowContent.addWidgetLabel("hframe2","size",_("size: ")+FilesEngine::printSize(file.getSize()));
		
		if(file.getSymlink()!="")
		{
			windowContent.addWidgetHorizontalFrame("vframe1","hframe3");
			windowContent.addWidgetLabel("hframe3","symlink",_("symlink: ")+file.getSymlink());
		}
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe4");
		windowContent.addWidgetLabel("hframe4","mime",_("mime: ")+file.getMime());
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe5");
		windowContent.addWidgetLabel("hframe5","owner",_("owner: ")+file.getUser());		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe6");
		windowContent.addWidgetLabel("hframe6","group",_("group: ")+file.getGroup());
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe7");
		windowContent.addWidgetLabel("hframe7","permissions",_("permissions: ")+FilesEngine::permissionsToString(file.getMode()));
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe8");
		windowContent.addWidgetLabel("hframe8","atime",_("access time: ")+FilesEngine::convertTimeToString(file.getAccessTime()));

		windowContent.addWidgetHorizontalFrame("vframe1","hframe9");
		windowContent.addWidgetLabel("hframe9","mtime",_("modification time: ")+FilesEngine::convertTimeToString(file.getModificationTime()));

		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetLabel("hframe10","ctime",_("creation time: ")+FilesEngine::convertTimeToString(file.getCreationTime()));

		
		/*TODO*/
		
		
		windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
		windowContent.addWidgetButton("hframe10","buttonOk",_("close"),"ok");	
	
		map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);

	
		if(isCanceledExecution())		
		return;
		
	}	
}



void CmdClipboardCopy::execute()
{
	fileList->getFileListIcons()->clipboardCopy();
}

void CmdClipboardCut::execute()
{
	fileList->getFileListIcons()->clipboardCopy(true);
}

void CmdClipboardPaste::execute()
{
	fileList->getFileListIcons()->clipboardPaste();
}



void CmdOpenPath::execute()
{
	string state=getParam();
	if(state=="execute")
	{

		string data=fileList->getTextFieldData();
		path p=data;
		fileList->openDirectory(p);
		
	}
	else if(state=="switch")
	{
		
		vector<string>data;
		data.push_back(fileList->getCurrentDirectory().string());
		fileList->getFileListIcons()->setTextFieldList(data);
	}
}

void CmdRunCommand::execute()
{
	string state=getParam();
	if(state=="execute")
	{
		string data=fileList->getTextFieldData()+" &";
		system(data.c_str());
		
	}
	else if(state=="switch")
	{
		vector<string>data;
		data.push_back("");
		fileList->getFileListIcons()->setTextFieldList(data);
	}
}



void CmdOpenFlatDirectory::execute()
{
	string newParams="vfs=FlatDirectory:directory="+fileList->getCurrentDirectory().string();
	fileList->getFileListIcons()->createPanel("FileList",newParams);
}

void CmdOpenHistoryVfs::execute()
{
	fileList->getFileListIcons()->createPanel("FileList","vfs=History");
}

void CmdOpenSearchVfs::execute()
{
	
	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");		
		
	windowContent.addWidgetHorizontalFrame("vframe1","hframe2");
	windowContent.addWidgetTextField("hframe2","filePattern","",25);
	windowContent.addWidgetLabel("hframe2","filePatternLabel",_("<= file pattern"));
		
	windowContent.addWidgetHorizontalFrame("vframe1","hframe10");
	windowContent.addWidgetButton("hframe10","ok",_("ok"),"ok");
	windowContent.addWidgetButton("hframe10","cancel",_("cancel"),"cancel");	
	
		
	map<string,string>answer=Question::askCustom(fileList->getFileListIcons(),windowContent);

	if(answer["pressedButton"]=="ok")
	{
		string newParams="vfs=Search:directory="+fileList->getCurrentDirectory().string()+":filePattern="+answer["filePattern"];
		fileList->getFileListIcons()->createPanel("FileList",newParams);
	}


	
}

void CmdOpenArchiveVfs::execute()
{
	try
	{
	string newParams="vfs=Archive:file="+fileList->getCurrentFileIcon().getFile().getPath().string();
	fileList->getFileListIcons()->createPanel("FileList",newParams);
	}
	catch(...){}
}


vector<string> CmdRegexSelection::last;
vector<string> CmdRegexFilter::last;



void CmdRegexSelection::execute( )
{
	string state=getParam();
	if(state=="execute")
	{
		string data=fileList->getTextFieldData();
		
		if(data=="")
		return;
		
		if(find(last.begin(),last.end(),data)==last.end())
		{		
		last.insert(last.begin(),data);
		fileList->getFileListIcons()->setTextFieldList(last);
		}

		try
		{
			/*TODO perl etc*/
			boost::regex pattern(data); 
	
			vector<string>files=fileList->getAllFilesPaths();
			vector<string>::iterator iter;
	
			for(iter=files.begin();iter!=files.end();++iter)
			{
				string fileName=path(*iter).leaf();
		
				if(!boost::regex_match(fileName, pattern))
				{	
				fileList->removeFileIcon(path(*iter).string());					
				}			
			}
		}
		catch(boost::bad_expression &e)
		{
		Question::displayError(fileList->getFileListIcons(),"bad expression");
		}	
	}
	else if(state=="switch")
	{
		if(!last.size())
		{
			Settings &settings=singleton<Settings>::instance();
			last=settings.getTextFieldLast( "RegexSelection" );
		}		
		fileList->getFileListIcons()->setTextFieldList(last);
	}
}

void CmdRegexFilter::execute()
{
	string state=getParam();
	if(state=="execute")
	{
		string data=fileList->getTextFieldData();
		
		if(data=="")
		return;
		
		if(find(last.begin(),last.end(),data)==last.end())
		{		
		last.insert(last.begin(),data);
		fileList->getFileListIcons()->setTextFieldList(last);
		}
		
		try
		{
			/*TODO perl itd*/
			boost::regex pattern(data); 
	
			vector<string>files=fileList->getAllFilesPaths();
			vector<string>::iterator iter;
	
			for(iter=files.begin();iter!=files.end();++iter)
			{			
				string fileName=path(*iter).leaf();
				if(boost::regex_match(fileName, pattern))
				{	
					fileList->selectFileIcon(*iter);		
				}
			}
		}
		catch(boost::bad_expression &e)
		{
		Question::displayError(fileList->getFileListIcons(),"bad expression");
		}	
	}
	else if(state=="switch")
	{
		if(!last.size())
		{
			Settings &settings=singleton<Settings>::instance();
			last=settings.getTextFieldLast( "RegexFilter" );
		}		
		fileList->getFileListIcons()->setTextFieldList(last);
	}
}



void CmdAddDirToShortcuts::execute()
{
	Settings &settings=singleton<Settings>::instance();
	string param="vfs=Local:directory="+fileList->getCurrentDirectory().string();
	CommandInfo cmdInfo(fileList->getCurrentDirectory().string(),"alias");
	cmdInfo.setAlias("OpenLocalVfs");
	cmdInfo.updateDescription("open "+fileList->getCurrentDirectory().leaf(),settings.getLanguage());
	cmdInfo.setParam(param);
	cmdInfo.setRunAsThread(false);
	settings.setCommand(cmdInfo);
}

void CmdUmount::execute()
{
	if(system(string("umount " +fileList->getCurrentFileIcon().getFile().getPath().string()).c_str())!=0)
	throw ExceptionFileSystem();
}

void CmdCreateEmptyFile::execute()
{
	string fileName;
	
	try
	{
	fileName=Question::askString(fileList->getFileListIcons(),_("file name:"),"");
	}
	catch(...)
	{
		return;
	}
	
	if(!singleton<Settings>::instance().getCreateNewFileWithUTF8())		
	fileName=Iconv::fromUTF8(fileName);
	
	
	string cmd="cd "+fileList->getCurrentDirectory().string()+" && touch "+fileName;
	if(system(cmd.c_str())==0)
	{
		path newPath=fileList->getCurrentDirectory()/path(fileName);
		fileList->addFile(newPath.string());
		fileList->sort();
	}
	else
	throw ExceptionFileSystem();
	
	
	
}

