/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef FOXCONFIGURATOR_HPP
#define FOXCONFIGURATOR_HPP

#include <fx.h>
#include "Settings.hpp"
#include "FoxIconManager.hpp"
#include "SettingsObservator.hpp"


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class FoxConfigurator : public FXDialogBox
{

FXDECLARE(FoxConfigurator);

public:
    
    /**
     * Default constructor.
     */
    FoxConfigurator( void ):
    settings(singleton<Settings>::instance()),
    foxIconManager(FoxSingleton<FoxIconManager>::instance(NULL)){}

    /**
     * Constructor.
     * @param owner - pointer to owner window.
     */
    FoxConfigurator ( FXWindow * owner );
    
    /**
     * Destructor.
     */
    virtual ~FoxConfigurator( void );
    
	enum
	{
	ID_MAINWINDOWHEIGHT=FXDialogBox::ID_LAST,
	ID_MAINWINDOWWIDTH,
	ID_ACTIVE_COMMAND_CHANGE,
	ID_REMOVE_COMMAND,
	ID_NEW_COMMAND,   
	ID_NEW_COMMAND_PYTHON_PLUGIN,
	ID_ACTIVE_FILETYPE_CHANGE,
	ID_ADD_FILETYPE,
	ID_REMOVE_FILETYPE,
	ID_FILETYPE_WIZARD,
	ID_UP_COMMAND,
	ID_DOWN_COMMAND,
	ID_ADD_COMMAND_ADDITIONAL,
	ID_DEL_COMMAND_ADDITIONAL,
	ID_CHOOSE_COLOR,
	ID_CHOOSE_BACKCOLOR,
	ID_REMOVE_BACK_COLOR,
	ID_REMOVE_COLOR,
	ID_COLORS,
	ID_CHOOSE_FONT1,
	ID_CHOOSE_FONT2,
	ID_CHOOSE_FONT3,
	ID_UP_VFS_HEADER,
	ID_DOWN_VFS_HEADER,
	ID_VFS_CHANGE,
	ID_ADD_HEADER,
	ID_DEL_HEADER,
	ID_TOOLBAR_CHANGE,
	
	ID_UP_BUTTON_COMMAND,
	ID_DOWN_BUTTON_COMMAND,
	ID_UP_SHUTTER_COMMAND,
	ID_DOWN_SHUTTER_COMMAND,
	
	ID_ADD_TOOLBAR_COMMAND,
	ID_DEL_TOOLBAR_COMMAND,
	ID_SHUTTER_CHANGE,
	ID_ADD_SHUTTER_COMMAND,
	ID_DEL_SHUTTER_COMMAND,	
	ID_NEW_SHUTTER,
	ID_REMOVE_SHUTTER,
	ID_SET_DEFAULT_SHUTTER,
	};
    
	long onChange (FXObject * sender, FXSelector, void *);
	long onActiveCommandChange (FXObject * sender, FXSelector, void *);
	long onNewCommand (FXObject * sender, FXSelector, void *);
	long onRemoveCommand (FXObject * sender, FXSelector, void *);
	long onActiveFileTypeChange (FXObject * sender, FXSelector, void *);
	long onAddFileType (FXObject * sender, FXSelector, void *);
	long onRemoveFileType (FXObject * sender, FXSelector, void *);
	long onFileTypeWizard (FXObject * sender, FXSelector, void *);
	long onUpDownAdditionalCommand(FXObject * sender, FXSelector, void *);
	long onAddAdditionalCommand(FXObject * sender, FXSelector, void *);
	long onDelAdditionalCommand(FXObject * sender, FXSelector, void *);	
	long onChooseColor ( FXObject * sender, FXSelector sel, void * );
	long onRemoveColor ( FXObject * sender, FXSelector sel, void * );
	long onRemoveBackColor ( FXObject * sender, FXSelector sel, void * );
	long onColorChanged ( FXObject * sender, FXSelector sel, void * );
	long onChooseFont ( FXObject * sender, FXSelector sel, void * );
	long onActiveVfsChange( FXObject * sender, FXSelector sel, void * );
	long onUpDownVfsHeader( FXObject * sender, FXSelector sel, void * );
	long onAddDelHeader( FXObject * sender, FXSelector sel, void * );
	long onUpDownShuttersAndToolbars(FXObject * sender, FXSelector sel, void * );
	long onShutterChange(FXObject * sender, FXSelector sel, void * );
	long onToolbarChange(FXObject * sender, FXSelector sel, void * );
	long onAddRemoveShutterCommand(FXObject * sender, FXSelector sel, void * );
	long onAddRemoveToolbarCommand(FXObject * sender, FXSelector sel, void * ); 
	long onNewShutter(FXObject * sender, FXSelector sel, void * ); 
	long onRemoveShutter(FXObject * sender, FXSelector sel, void * ); 
	long onSetDefaultShutter(FXObject * sender, FXSelector sel, void * ); 
	long onCmdClose(FXObject * sender, FXSelector sel, void * ); 
	
private:
	
	void setActiveCommand(string);
	void saveActiveCommand();

	void setActiveFileType(string);
	void saveActiveFileType();
	
	void saveActiveVfs();
	void setActiveVfs(string);
	
	void setAllColor( FXButton* button, FXColor color );
	void setAllStandardColor( FXButton* button);
	
	void setActiveShutter(string);
	void saveActiveShutter();
	
	void setActiveToolbar(string);
	void saveActiveToolbar();
	
	void saveSpeedAndDelays();
	void saveLookAndFeel();
	void saveMainSettings();
	void saveCharset();
	
	void setupLookAndFeel();
	void setupMainSettings();
	void setupFileType();
	void setupCommands();
	void setupSpeedAndDelays();
	void setupVfs();
	void setupMenuAndButtons();
	void setupCharset();
	void setupColors();
	void setupFont( int nr );
	
	
	
	FoxIconManager &foxIconManager;
	Settings &settings;
	vector<string> icons;
	vector<string> commands;
	
	
	FXSpinner* mainwindow_width,*mainwindow_height,*thumbsize,*historySize;
	FXListBox* commandsCombo,*fileTypeList,*availableMimeList,*defaultCommandList;
	FXListBox* iconsList,*fileTypeIconsList;
	FXTextField *shortcutKey,*commandExecute, *commandDescription,*newCommandName, *commandGesture;
	FXLabel *commandType;
	
	string activeCommandName;
	string activeFileTypeName;
	string activeVfs;
	string activeShutter;
	string activeToolbar;
	
	FXButton *removeCommandButton;
	map<string,int>iconNames;
        vector<string>iconNamesVector;
        FXApp *app;
        FXList *additionalCommands;
        FXList *additionalCommandsAvailable;
        FXButton *colorbutton;
   	FXButton *backcolorbutton;
	FXColorDialog *colordlg;
	FXVerticalFrame *buttons;
	FXSwitcher *switcher;
	FXListBox* placement,*iconsTheme;
	
	
	FXuint typingSpeed;
	FXuint clickSpeed;
	FXuint scrollSpeed;
	FXuint scrollDelay;
	FXuint blinkSpeed;
	FXuint animSpeed;
	FXuint menuPause;
	FXuint tooltipPause;
	FXuint tooltipTime;
	FXint dragDelta;
	FXint wheelLines;	
	
	FXDataTarget target_typingspeed;
	FXDataTarget target_clickspeed;
	FXDataTarget target_scrollspeed;
	FXDataTarget target_scrolldelay;
	FXDataTarget target_blinkspeed;
	FXDataTarget target_animspeed;
	FXDataTarget target_menupause;
	FXDataTarget target_tooltippause;
	FXDataTarget target_tooltiptime;
	FXDataTarget target_dragdelta;
	FXDataTarget target_wheellines;
	
	
	
	FXColor base;
	FXColor border;
	FXColor back;
	FXColor fore;
	FXColor selback;
	FXColor selfore;
	FXColor tipback;
	FXColor tipfore;
	FXColor menuback;
	FXColor menufore;
	FXColor hilite;
	FXColor shadow;
	FXColor main;
			
	FXDataTarget target_base;
	FXDataTarget target_back;
	FXDataTarget target_border;
	FXDataTarget target_fore;
	FXDataTarget target_hilite;
	FXDataTarget target_shadow;
	FXDataTarget target_selfore;
	FXDataTarget target_selback;
	FXDataTarget target_tipfore;
	FXDataTarget target_tipback;
	FXDataTarget target_menufore;
	FXDataTarget target_menuback;
	
	FXDataTarget target_main;
	
	
	FXToolTip *tooltip;
	FXTabBook *tabbook;
	FXTabItem *tabitem;
	FXVerticalFrame *tabframe;
	FXVerticalFrame *mainframe;
	FXVerticalFrame *menuframe;
	FXHorizontalFrame *labeltextframe1;
	FXHorizontalFrame *labeltextframe2;
	FXHorizontalFrame *textframe1;
	FXHorizontalFrame *textframe2;
	FXHorizontalFrame *tabsubframe;
	FXGroupBox *grpbox1;
	FXGroupBox *grpbox2;
	FXLabel *label1;
	FXLabel *label2;
	FXLabel *label3;
	FXLabel *label4;
	FXLabel *label5;
	FXLabel *menulabels[ 6 ];
	FXTextField *textfield1;
	FXButton *button1;
	FXButton *fontbutton[ 3 ];
	FXSeparator *sep1;
	FXSeparator *sep2;
	FXSeparator *sep3;
	
	FXFont *font[ 3 ];
	FXTextField *leftdir, *rightdir, *defaultdir;
	FXListBox *vfsList;
	FXList* headersList, *availableHeadersList;
	
	FXListBox *toolbarList,*shutterList;
	FXList *buttonsList,*additionalCommandsAvailableForButtons,*shutterCommands;
	FXTextField *newShutterEdit;
	
	FXCheckButton *showHiddenFilesCheck,*markInaccessibleFoldersCheck,*markInaccessibleFilesCheck,*createNewFileWithUTF8Check;
	FXCheckButton *checkForceReload,*checkNeedSelectedFile, *saveWindowSizeCheck, *saveOriginalCharsetOnRenameCheck;
	
	int initialHeight;
	int initialWidth;
	
	FXTextField *charsetField;


};

#endif
