/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include "FoxStartup.hpp"
#include "FoxMainWindow.hpp"
#include "AutoConfigurator.hpp"
#include "FoxAutoConfigurator.hpp"
#include "Settings.hpp"
#include "Exceptions.hpp"

#include "config.h"
#include "gettext.h"
#define _(STRING)    gettext(STRING)

int FoxStartup::start(int argc, char * argv[])
{

	fxTraceLevel = 0;
	
	FXApp application( "nao2", "nao2" );
	application.init ( argc, argv );
	
	
	AutoConfigurator &autoConfigurator=singleton<AutoConfigurator>::instance();
	
	bool firstRun=false;
	try
	{
		if(autoConfigurator.isFirstRun())
		{
			autoConfigurator.firstRunConfiguration();
			firstRun=true;
		}
	}
	catch(ExceptionVersionMismatch &e)
	{
		autoConfigurator.clearOldConfiguration();
		autoConfigurator.firstRunConfiguration();
		firstRun=true;
	}
	
	FXToolTip *tooltip=new FXToolTip ( &application, TOOLTIP_NORMAL );
	
	Settings &settings=singleton<Settings>::instance();
	
	application.setTypingSpeed( settings.getTypingSpeed() );
	application.setClickSpeed(  settings.getClickSpeed() );
	application.setScrollSpeed( settings.getScrollSpeed() );
	application.setScrollDelay( settings.getScrollDelay() );
	application.setBlinkSpeed( settings.getBlinkSpeed() );
	application.setAnimSpeed( settings.getAnimSpeed() );
	application.setMenuPause( settings.getMenuPause() );
	application.setTooltipPause( settings.getTooltipPause() );
	application.setTooltipTime( settings.getTooltipTime() );
	application.setDragDelta( settings.getDragDelta() );
	application.setWheelLines( settings.getWheelLines() );
	
	application.setBaseColor(fxcolorfromname(settings.getBaseColor().c_str()));
	application.setBackColor(fxcolorfromname(settings.getBackColor().c_str()));
	application.setBorderColor(fxcolorfromname(settings.getBorderColor().c_str()));
	application.setForeColor(fxcolorfromname(settings.getForeColor().c_str()));
	application.setHiliteColor(fxcolorfromname(settings.getHiliteColor().c_str()));
	application.setShadowColor(fxcolorfromname(settings.getShadowColor().c_str()));
	application.setSelforeColor(fxcolorfromname(settings.getSelforeColor().c_str()));
	application.setSelbackColor(fxcolorfromname(settings.getSelbackColor().c_str()));
	application.setTipforeColor(fxcolorfromname(settings.getTipforeColor().c_str()));
	application.setTipbackColor(fxcolorfromname(settings.getTipbackColor().c_str()));
	application.setSelMenuTextColor(fxcolorfromname(settings.getMenuforeColor().c_str()));
	application.setSelMenuBackColor(fxcolorfromname(settings.getMenubackColor().c_str()));

	/*TODO if font doesnt exist*/	
	application.setNormalFont(new FXFont( &application, settings.getFontDescription("normal").c_str() ));
	tooltip->setFont(new FXFont( &application, settings.getFontDescription("normal").c_str() ));
	
	
	if(firstRun)
	{
		application.create ();
		
		
		if(settings.isVersionMismatch())
		FXMessageBox::warning (&application, MBOX_OK, _("warning"), _("Found old configuration file, incomaptible with new version. The old configuration will be deleted."));
		
		
		
		if(FXMessageBox::question (&application, MBOX_YES_NO, _("question"), _("Would you like to associate programs with file types?\nIt is recommended to do this now."))==MBOX_CLICKED_YES)
		{
		
		
		FoxAutoConfigurator foxAutoConfigurator(&application);
		foxAutoConfigurator.setMimeAssociations();
				
		}
	
		FoxMainWindow *mainWindow=new FoxMainWindow ( &application );
		mainWindow->create();
		return application.run ();
	}
	else
	{
		new FoxMainWindow ( &application );
		application.create ();
		return application.run ();
	}
	
}
