/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "VfsArchive.hpp"
#include "FilesEngine.hpp"
#include "Settings.hpp"


#include <boost/lexical_cast.hpp>
using boost::lexical_cast;

REGISTER_FACTORABLE(VfsArchive);

VfsArchive::VfsArchive() : listed(false)
{
}


VfsArchive::~VfsArchive()
{
}

Directory VfsArchive::openDir(path & p, Cancelable * cancelable)
{
	if(!listed)
	{
	listFiles();
	listed=true;
	}
	
	iter=allFiles.begin();
	if(p.string()=="")
	p="/";
	
	Directory directory(this,NULL,p);
	return directory;
	

}

void VfsArchive::changeOwnerRecursive(path & source, string user)
{
	throw ExceptionNotImplemented();
}

File VfsArchive::readFullFile(File file)
{
	if(!file.isDirectory())
	file.setMime(mime.getFileMime(file.getPath(),false));
	else
	file.setMime("special/directory");
	
	return file;
}

void VfsArchive::rename(path & source, path & destination)
{
	throw ExceptionNotImplemented();
}


void VfsArchive::removeDirectory(path & source)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::removeFile(path & source)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::chmodRecursive(path & source, unsigned int mode)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::chmod(path & source, unsigned int mode)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::init(Panel * _panel, QuestionManager * _questionManager)
{
	panel=_panel;
	questionManager=_questionManager;
	params=FilesEngine::parseParam(panel->getParam());
	
	panel->setLabel(path(params["file"]).leaf()+":","archive_small");
	archiveFileName=params["file"];
	//setHeaders(settings.getHeaders("Archive"));	
	
}

string VfsArchive::getType()
{
	return "remote";
}

double VfsArchive::getTotalSize(path & p, Cancelable * cancelable)
{
	
}

path VfsArchive::symlink(const path & file)
{
	return "";
}

void VfsArchive::copy(Vfs * vfs2, path source, path destination, Status * status, Cancelable * cancelable)
{

	File file;
	file.setName(source.leaf());
	file.setParentDirectory(source.branch_path());


	if(vfs2->getType()=="Local")
	{
	copyFromFile(file,destination,status,cancelable);
	}
	else
	{
 	copyToFile(file,destination,status,cancelable);
	}


}



void VfsArchive::decompressArchive()
{
	string command=decompress + " " + archiveFileName;
	system(command.c_str());
}

void VfsArchive::compressArchive()
{
	string command=compress + " " + archiveFilenameDecompressed;
	system(command.c_str());
}



void VfsArchive::remove(path & source)
{

	string fileName=source.string();
	fileName=fileName.substr(1,fileName.size()-1);
	
	string command;
	
		decompressArchive();		
		
		command="tar --delete -f " + archiveFilenameDecompressed + " " +fileName;		
		system(command.c_str());
		
		compressArchive();		
		
		for(iter=allFiles.begin();iter!=allFiles.end();++iter)
		{
			if(iter->getPath()==source)
			{
			allFiles.erase(iter);
			break;
			}
		}		

}

void VfsArchive::copyToFile(File & source, path & destination, Status * status, Cancelable * cancelable)
{

	string srcDir=source.getPath().branch_path().string();
	string command;
	
	
	decompressArchive();
	
	if(destination.branch_path().string()=="/")
	{			
		command="cd \"" +srcDir + "\" && tar rf \"" + archiveFilenameDecompressed + "\" \"" +source.getPath().leaf()+"\"";
		system(command.c_str());		
	}
	else
	{
		path dstPath=createTmpDir();
		FilesEngine::createDir(dstPath);
		
		path::iterator pathIter;
		path srcPath=srcDir;
		path subDir=dstPath;
		
		path dd=destination.branch_path();
		for(pathIter=dd.begin();pathIter!=dd.end();++pathIter)
		{
			if((*pathIter)=="/")
			continue;
			
			subDir=subDir/path(*pathIter);
			FilesEngine::createDir(subDir);
		}
		path ss=source.getPath();
		FilesEngine::copy(ss,subDir);
		
		command="cd \"" +dstPath.string() + "\" && tar rf \"" + archiveFilenameDecompressed + "\" \"" +dd.string().substr(1,dd.string().size()-1)+"/"+source.getPath().leaf()+"\"";
		system(command.c_str());
		FilesEngine::remove(dstPath);		
		
	
	}
	
	File file;
	file.setName(source.getPath().leaf());
	file.setParentDirectory(destination.branch_path().string());
	allFiles.push_back(file);
	
	compressArchive();

	
}

void VfsArchive::copyFromFile(File & source, path & destination, Status * status, Cancelable * cancelable)
{
	string dst=destination.branch_path().string();
	string src=source.getPath().string();
	src=src.substr(1,src.size()-1);
	
	path dstPath;
	
	if(source.getParentDirectory()!="/")
	{	
		dstPath=createTmpDir();
		FilesEngine::createDir(dstPath);	
	}
	
	
	string command="cd \""+dst+"\" && tar xf" + type + " \"" + params["file"] + "\" \"" + src +"\"";
	system(command.c_str());
	
	
	if(source.getParentDirectory()!="/")
	{
		path ss=path(dst)/source.getPath();
		path dd=destination.branch_path();
		FilesEngine::copy(ss,dd);
		FilesEngine::remove(dstPath);
	}

	
	
	
}

void VfsArchive::createDir(path p)
{
	throw ExceptionNotImplemented();
}

File VfsArchive::readFile(path file)
{

	for(iter=allFiles.begin();iter!=allFiles.end();++iter)
	{
		if(iter->getPath().string()==file.string())
		return *iter;
	}
	
	throw ExceptionReadFile();
}

void VfsArchive::closeDir(Directory * directory)
{
}

File VfsArchive::readDir(Directory * directory)
{

	string actualDirectory=directory->getDir().string();

	
	while(iter!=allFiles.end())
	{
		if(iter->getParentDirectory()==actualDirectory)
		{	
			File file=*iter;		
			++iter;
			return file;
		}
	++iter;
	}		    
	
	

	throw ExceptionReadDirEnd();
}

void VfsArchive::createSymlink(path & source, path & destination)
{
	throw ExceptionNotImplemented();
}

vector< string > VfsArchive::getGroups()
{
	vector<string>g;
	return g;
}

vector< string > VfsArchive::getUsers()
{
	vector<string>g;
	return g;
}

void VfsArchive::changeGroup(path & source, string group)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::changeGroupRecursive(path & source, string group)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::changeOwner(path & source, string user)
{
	throw ExceptionNotImplemented();
}

void VfsArchive::listFiles()
{
	string archiveFilename=params["file"];

	vector<string>allPaths;
	vector<string>::iterator iter;
	
	string  command;
	
	if ((archiveFilename.length () >= 6 && archiveFilename.substr (archiveFilename.length () - 6, 6) == "tar.gz") ||(archiveFilename.length () >= 3 && archiveFilename.substr (archiveFilename.length () - 3, 3) == "tgz"))
	type="z";
	else
	type="j";
	
	command="tar tf" + type + " " + archiveFilename;
	
	
	pipe = popen(command.c_str(), "r");
	
	char readbuf[1024];
	
	if (pipe != NULL)
	{
		string line;
		while (fgets (readbuf, sizeof (readbuf), pipe) != NULL)
		{	
		string path = readbuf;
		path = "/"+path.substr (0, path.length () - 1);
		allPaths.push_back(path);		
		}
	
		pclose (pipe);
	}
	
	iter=allPaths.begin();

	while(iter!=allPaths.end())
	{
		string name=*iter;

		
		bool isDir=false;
		if(name.substr(name.size()-1,1)=="/")
		{
			isDir=true;
		}
	
		string comparePath=name;
		if(isDir)
		comparePath=name.substr(0,name.size()-1);
		
		int pos=comparePath.rfind("/");
		string parentDir=comparePath.substr(0,pos);
		string fileName=comparePath.substr(pos+1,comparePath.size()-pos-1);
		
		if(parentDir=="")
		parentDir="/";
		
		File file;
		file.setName(fileName);
		file.setParentDirectory(parentDir);
		
		if(isDir)
		file.setDirectory();	
		
		allFiles.push_back(file);
		++iter;
	}	
		
		
		
		
		if(type=="z")
		{
			decompress="gunzip";
			compress="gzip";
			archiveFilenameDecompressed=archiveFileName.substr(0,archiveFileName.length()-3);
		}
		else //bzip2
		{
			decompress="bunzip2";
			compress="bzip2";
			archiveFilenameDecompressed=archiveFileName.substr(0,archiveFileName.length()-4);
		}	
		


}

path VfsArchive::createTmpDir()
{
	path tmpfile;
		path p=path("/tmp/nao");
		if(!FilesEngine::exists(p))
		FilesEngine::createDir(p);
	
		srand((unsigned)time(0));		
		
		do
		{
			int randomInteger = rand(); 	
			string randomPart=lexical_cast<string>(randomInteger);	
			tmpfile=path("/tmp/nao")/path(randomPart);	
		}
		while(FilesEngine::exists(tmpfile));	
		
	return tmpfile;	
}






